@@ -1,5 +1,5 @@
 /***************************************************************************
-    Copyright (C) 2009 Robby Stephenson <robby@periapsis.org>
+    Copyright (C) 2009-2013 Robby Stephenson <robby@periapsis.org>
  ***************************************************************************/
 
 /***************************************************************************
@@ -37,6 +37,7 @@
 #include <klocale.h>
 #include <kio/job.h>
 #include <kio/jobuidelegate.h>
+#include <KLineEdit>
 
 #include <QLabel>
 #include <QFile>
@@ -50,10 +51,11 @@
 #endif
 
 namespace {
-  static const char* FREEBASE_QUERY_URL = "http://api.freebase.com/api/service/mqlread";
-  static const char* FREEBASE_IMAGE_URL = "http://api.freebase.com/api/trans/image_thumb";
-  static const char* FREEBASE_BLURB_URL = "http://api.freebase.com/api/trans/blurb";
+  static const char* FREEBASE_QUERY_URL = "https://www.googleapis.com/freebase/v1/mqlread";
+  static const char* FREEBASE_IMAGE_URL = "https://usercontent.googleapis.com/freebase/v1/image";
+  static const char* FREEBASE_BLURB_URL = "https://www.googleapis.com/freebase/v1/topic";
   static const char* FREEBASE_VIEW_URL  = "http://www.freebase.com/view";
+  static const char* FREEBASE_API_KEY   = "AIzaSyBdsa_DEGpDQ6PzZyYHHHokRIBY8thOdUQ";
   static const int   FREEBASE_RETURNS_PER_REQUEST = 25;
 }
 
@@ -61,7 +63,9 @@ using namespace Tellico;
 using Tellico::Fetch::FreebaseFetcher;
 
 FreebaseFetcher::FreebaseFetcher(QObject* parent_)
-    : Fetcher(parent_), m_job(0), m_started(false) {
+    : Fetcher(parent_)
+    , m_started(false)
+    , m_apiKey(QLatin1String(FREEBASE_API_KEY)) {
 }
 
 FreebaseFetcher::~FreebaseFetcher() {
@@ -85,18 +89,22 @@ bool FreebaseFetcher::canSearch(FetchKey k) const {
   return k == Title || k == Person || k == ISBN || k == LCCN;
 }
 
-void FreebaseFetcher::readConfigHook(const KConfigGroup&) {
+void FreebaseFetcher::readConfigHook(const KConfigGroup& config_) {
+  m_apiKey = config_.readEntry("API Key", FREEBASE_API_KEY);
 }
 
 void FreebaseFetcher::search() {
-  m_started = true;
   m_cursors.clear();
-  doSearch();
+  continueSearch();
 }
 
 void FreebaseFetcher::continueSearch() {
   m_started = true;
+  m_hasMoreResults = false;
   doSearch();
+  if(m_jobs.isEmpty()) {
+    stop();
+  }
 }
 
 void FreebaseFetcher::doSearch() {
@@ -142,23 +150,18 @@ void FreebaseFetcher::doSearch() {
     return;
   }
 
-  QVariantMap httpQuery;
+  QJson::Serializer serializer;
+  foreach(const QVariant& queryVariant, queries) {
+    QVariantMap query = queryVariant.toMap();
+    Q_ASSERT(!query.isEmpty());
 
-  // make sure if we have cursors for every query
-  while(m_cursors.count() < queries.count()) {
-    m_cursors.append(QVariant(true));
-  }
-
-  for(int i = 0; i < queries.size(); ++i) {
+    const uint queryHash = qHash(serializer.serialize(query));
     // we skip the query if the cursor is valid and == false
-    if(m_cursors.at(i).type() == QVariant::Bool && !m_cursors.at(i).toBool()) {
-      myDebug() << "skipping query" << i;
+    const QVariant cursor = m_cursors.value(queryHash);
+    if(cursor.type() == QVariant::Bool && !cursor.toBool()) {
       continue;
     }
 
-    QVariantMap query = queries.at(i).toMap();
-    Q_ASSERT(!query.isEmpty());
-
     // grab all properties at the entity level
     query.insert(QLatin1String("*"), QVariantList());
     query.insert(QLatin1String("limit"), FREEBASE_RETURNS_PER_REQUEST);
@@ -171,38 +174,52 @@ void FreebaseFetcher::doSearch() {
     query.insert(QLatin1String("/common/topic/image"), id_query);
     query.insert(QLatin1String("/common/topic/article"), id_query);
 
-    QVariantMap envelope;
-    envelope.insert(QLatin1String("query"), QVariantList() << query);
-    envelope.insert(QLatin1String("cursor"), m_cursors.at(i));
-    httpQuery.insert(QString::fromLatin1("q%1").arg(i), envelope);
-  }
+    const QByteArray query_string = serializer.serialize(QVariantList() << query);
+//    myDebug() << "query:" << query_string;
 
-  QJson::Serializer serializer;
-  QByteArray query_string = serializer.serialize(httpQuery);
-//  myDebug() << "query:" << query_string;
-
-  KUrl url(FREEBASE_QUERY_URL);
-  if(query_string.length() < 2048) {
-    url.addQueryItem(QLatin1String("queries"), QString::fromUtf8(query_string));
-    m_job = KIO::storedGet(url, KIO::NoReload, KIO::HideProgressInfo);
-  } else {
-    query_string.prepend("queries=");
-    m_job = KIO::storedHttpPost(query_string, url, KIO::HideProgressInfo);
-  }
-  m_job->ui()->setWindow(GUI::Proxy::widget());
-  connect(m_job, SIGNAL(result(KJob*)),
+    KUrl url(FREEBASE_QUERY_URL);
+    url.addQueryItem(QLatin1String("key"), QLatin1String(FREEBASE_API_KEY));
+    if(cursor.type() == QVariant::String) {
+      url.addQueryItem(QLatin1String("cursor"), cursor.toString());
+//      myDebug() << "using cursor" << cursor;
+    } else {
+      url.addQueryItem(QLatin1String("cursor"), QString());
+    }
+//    if(query_string.length() < 2048) {
+//    url.addQueryItem(QLatin1String("queries"), QString::fromUtf8(query_string));
+      url.addQueryItem(QLatin1String("query"), QString::fromUtf8(query_string));
+      QPointer<KIO::StoredTransferJob> job = KIO::storedGet(url, KIO::NoReload, KIO::HideProgressInfo);
+//    } else {
+//      query_string.prepend("queries=");
+//      job = KIO::storedHttpPost(query_string, url, KIO::HideProgressInfo);
+//    }
+//    myDebug() << url.url();
+    job->setProperty("freebase_query", queryHash);
+    job->ui()->setWindow(GUI::Proxy::widget());
+    connect(job, SIGNAL(result(KJob*)),
           SLOT(slotComplete(KJob*)));
+    m_jobs << job;
+  }
 #endif
 }
 
+void FreebaseFetcher::endJob(KIO::StoredTransferJob* job_) {
+  m_jobs.removeOne(job_);
+  if(m_jobs.isEmpty())  {
+    stop();
+  }
+}
+
 void FreebaseFetcher::stop() {
   if(!m_started) {
     return;
   }
-  if(m_job) {
-    m_job->kill();
-    m_job = 0;
+  foreach(QPointer<KIO::StoredTransferJob> job, m_jobs) {
+    if(job) {
+      job->kill();
+    }
   }
+  m_jobs.clear();
   m_started = false;
   emit signalDone(this);
 }
@@ -222,6 +239,7 @@ Tellico::Data::EntryPtr FreebaseFetcher::fetchEntryHook(uint uid_) {
     imageUrl.addPath(image_id);
     imageUrl.addQueryItem(QLatin1String("maxwidth"), QLatin1String("200"));
     imageUrl.addQueryItem(QLatin1String("maxheight"), QLatin1String("200"));
+//    myDebug() << "image url:" << imageUrl;
     const QString id = ImageFactory::addImage(imageUrl, true);
     if(id.isEmpty()) {
       message(i18n("The cover image could not be loaded."), MessageHandler::Warning);
@@ -251,13 +269,26 @@ Tellico::Data::EntryPtr FreebaseFetcher::fetchEntryHook(uint uid_) {
     if(article_id.startsWith(QLatin1Char('/'))) {
       KUrl articleUrl(FREEBASE_BLURB_URL);
       articleUrl.addPath(article_id);
-      articleUrl.addQueryItem(QLatin1String("maxlength"), QLatin1String("1000"));
-      articleUrl.addQueryItem(QLatin1String("break_paragraphs"), QLatin1String("true"));
-      const QString output = FileHandler::readTextFile(articleUrl, true, true);
-      if(output.isEmpty()) {
+      articleUrl.addQueryItem(QLatin1String("limit"), QLatin1String("1"));
+      articleUrl.addQueryItem(QLatin1String("filter"), QLatin1String("/common/document/text"));
+      articleUrl.addQueryItem(QLatin1String("key"), QLatin1String(FREEBASE_API_KEY));
+//      articleUrl.addQueryItem(QLatin1String("maxlength"), QLatin1String("1000"));
+//      articleUrl.addQueryItem(QLatin1String("break_paragraphs"), QLatin1String("true"));
+      const QByteArray data = FileHandler::readDataFile(articleUrl, true);
+      if(data.isEmpty()) {
         entry->setField(article_field, QString());
+#ifdef HAVE_QJSON
       } else {
-        entry->setField(article_field, output);
+        QJson::Parser parser;
+        const QVariant response = parser.parse(data);
+        const QVariantMap articleMap = response.toMap()
+              .value(QLatin1String("property")).toMap()
+              .value(QLatin1String("/common/document/text")).toMap();
+        const QVariantList valueList = articleMap.value(QLatin1String("values")).toList();
+        if(!valueList.isEmpty()) {
+          entry->setField(article_field, value(valueList.first().toMap(), "value"));
+        }
+#endif
       }
     }
   }
@@ -281,20 +312,21 @@ Tellico::Fetch::FetchRequest FreebaseFetcher::updateRequest(Data::EntryPtr entry
   return FetchRequest();
 }
 
-void FreebaseFetcher::slotComplete(KJob*) {
+void FreebaseFetcher::slotComplete(KJob* job_) {
+  KIO::StoredTransferJob* job = static_cast<KIO::StoredTransferJob*>(job_);
 #ifdef HAVE_QJSON
 //  myDebug();
 
-  if(m_job->error()) {
-    m_job->ui()->showErrorMessage();
-    stop();
+  if(job->error()) {
+    job->ui()->showErrorMessage();
+    endJob(job);
     return;
   }
 
-  QByteArray data = m_job->data();
+  QByteArray data = job->data();
   if(data.isEmpty()) {
     myDebug() << "no data";
-    stop();
+    endJob(job);
     return;
   }
 
@@ -303,59 +335,50 @@ void FreebaseFetcher::slotComplete(KJob*) {
     QFile f(QString::fromLatin1("/tmp/test.json"));
     if(f.open(QIODevice::WriteOnly)) {
       QTextStream t(&f);
-      t.setCodec(QTextCodec::codecForName("UTF-8"));
+      t.setCodec("UTF-8");
       t << data;
     }
     f.close();
 #endif
 
-  // since the fetch is done, don't worry about holding the job pointer
-  m_job = 0;
-
   QJson::Parser parser;
   QVariant response = parser.parse(data);
   if(response.isNull()) {
     myDebug() << "no response";
-    stop();
+    endJob(job);
     return;
   }
 
-  m_hasMoreResults = false;
+  const uint queryHash = job->property("freebase_query").toUInt();
+
   QVariantList resultList;
 
   const QVariantMap responseMap = response.toMap();
   // check response code to see if everything was ok
-  if(value(responseMap, "code") == QLatin1String("/api/status/ok")) {
-    // the result objects are in outer envelopes called q0, q1, q3, etc...
-    for(int i = 0; i < m_cursors.count(); ++i) {
-      QVariant queryResult = responseMap.value(QString::fromLatin1("q%1").arg(i));
-      if(!queryResult.isNull()) {
-        m_cursors[i] = queryResult.toMap().value(QLatin1String("cursor"));
-        // there are more results if the cursor value is a string
-        m_hasMoreResults = m_hasMoreResults || m_cursors.at(i).type() == QVariant::String;
+  if(responseMap.contains(QLatin1String("result"))) {
+    m_cursors[queryHash] = responseMap.value(QLatin1String("cursor"));
+    // there are more results if the cursor value is a string
+    m_hasMoreResults = m_hasMoreResults || m_cursors.value(queryHash).type() == QVariant::String;
+//    myDebug() << m_hasMoreResults << queryHash;
 
-        QVariant resultThing = queryResult.toMap().value(QLatin1String("result"));
-        if(resultThing.canConvert(QVariant::List)) {
-          resultList += resultThing.toList();
-        } else if(!resultThing.isNull()) {
-          resultList += resultThing.toMap();
-        }
-      }
+    const QVariant resultThing = responseMap.value(QLatin1String("result"));
+    if(resultThing.canConvert(QVariant::List)) {
+      resultList += resultThing.toList();
+    } else if(!resultThing.isNull()) {
+      resultList += resultThing.toMap();
     }
-  } else {
+  } else if(responseMap.contains(QLatin1String("error"))) {
     // we have an error!!!!!!!!!!!!!
-    QString msg = value(responseMap, "message");
-    if(msg.isEmpty()) {
-      msg = value(responseMap, "messages", "message");
-    }
+    const QString msg = value(responseMap, "error", "message");
     if(!msg.isEmpty()) {
       myDebug() << "message:" << msg;
+      message(msg, MessageHandler::Error);
     }
   }
 
   if(resultList.isEmpty()) {
-    myDebug() << "no results";
-    stop();
+//    myDebug() << "no results";
+    endJob(job);
     return;
   }
 
@@ -371,7 +394,6 @@ void FreebaseFetcher::slotComplete(KJob*) {
   QString output;
 
   foreach(const QVariant& result, resultList) {
-  //  myDebug() << "found result:" << result;
     QVariantMap resultMap = result.toMap();
 
     Data::EntryPtr entry(new Data::Entry(coll));
@@ -384,14 +406,32 @@ void FreebaseFetcher::slotComplete(KJob*) {
       case Data::Collection::Book:
       case Data::Collection::Bibtex:
         entry->setField(QLatin1String("pub_year"),  value(resultMap, "publication_date").left(4));
-        entry->setField(QLatin1String("isbn"),      value(resultMap, "isbn", "isbn"));
+        {
+          QString isbn = value(resultMap, "isbn", "isbn");
+          if(isbn.isEmpty()) {
+            isbn = value(resultMap, "/media_common/cataloged_instance/isbn13", "value");
+          }
+          entry->setField(QLatin1String("isbn"), isbn);
+        }
         entry->setField(QLatin1String("publisher"), value(resultMap, "publisher"));
         entry->setField(QLatin1String("pages"),     value(resultMap, "number_of_pages"));
-        entry->setField(QLatin1String("lccn"),      value(resultMap, "LCCN"));
+        {
+          QString lccn = value(resultMap, "LCCN");
+          if(lccn.isEmpty()) {
+            lccn = value(resultMap, "/media_common/cataloged_instance/lccn", "value");
+          }
+          entry->setField(QLatin1String("lccn"), lccn);
+        }
         entry->setField(QLatin1String("author"),    value(resultMap, "work:book", "author"));
         entry->setField(QLatin1String("editor"),    value(resultMap, "work:book", "editor"));
         entry->setField(QLatin1String("cr_year"),   value(resultMap, "work:book", "copyright_date").left(4));
-        entry->setField(QLatin1String("series"),    value(resultMap, "work:book", "part_of_series"));
+        {
+          const QString series = value(resultMap, "work:book", "part_of_series");
+          // some have duplicate values?
+          QStringList seriesList = FieldFormat::splitValue(series);
+          seriesList.removeDuplicates();
+          entry->setField(QLatin1String("series"), seriesList.join(FieldFormat::delimiterString()));
+        }
         entry->setField(QLatin1String("language"),  value(resultMap, "work:book", "original_language"));
         entry->setField(QLatin1String("pages"),     value(resultMap, "number_of_pages", "numbered_pages"));
         entry->setField(QLatin1String("genre"), FieldFormat::capitalize(value(resultMap, "book", "genre")));
@@ -399,6 +439,8 @@ void FreebaseFetcher::slotComplete(KJob*) {
           QString binding = value(resultMap, "binding");
           if(binding.toLower() == QLatin1String("hardcover")) {
             binding = i18n("Hardback");
+          } else if(binding.toLower() == QLatin1String("mass market paperback")) {
+            binding = i18n("Paperback");
           } else {
             binding = FieldFormat::capitalize(binding);
           }
@@ -545,7 +587,7 @@ void FreebaseFetcher::slotComplete(KJob*) {
         break;
     }
 
-    // the the image and article ids. The actual content gets loaded in fetchEntryHook()
+    // set the image and article ids. The actual content gets loaded in fetchEntryHook()
     entry->setField(QLatin1String("cover"), value(resultMap, "/common/topic/image", "id"));
     if(type == Data::Collection::Album && entry->field(QLatin1String("cover")).isEmpty()) {
       // musical releases can have the image from the album
@@ -566,7 +608,7 @@ void FreebaseFetcher::slotComplete(KJob*) {
     emit signalResultFound(r);
   }
 #endif
-  stop(); // required
+  endJob(job);
 }
 
 Tellico::Fetch::ConfigWidget* FreebaseFetcher::configWidget(QWidget* parent_) const {
@@ -617,13 +659,10 @@ QVariantList FreebaseFetcher::bookQueries() const {
   page_query.insert(QLatin1String("optional"), QLatin1String("optional"));
   query.insert(QLatin1String("number_of_pages"), page_query);
 
-  QVariantMap isbn_query;
-  isbn_query.insert(QLatin1String("type"), QLatin1String("/book/isbn"));
-  isbn_query.insert(QLatin1String("isbn"), QVariantList());
-  if(request().key != ISBN) {
-    isbn_query.insert(QLatin1String("optional"), QLatin1String("optional"));
-  }
-  query.insert(QLatin1String("isbn"), isbn_query);
+  QVariantMap media_query;
+  media_query.insert(QLatin1String("*"), QVariantList());
+  media_query.insert(QLatin1String("optional"), QLatin1String("optional"));
+  query.insert(QLatin1String("/media_common/cataloged_instance/isbn13"), media_query);
 
   QVariantList queries;
   switch(request().key) {
@@ -636,18 +675,22 @@ QVariantList FreebaseFetcher::bookQueries() const {
       // for now just check author, and need to use written_work topic_query
       {
         QVariantMap workQuery = query.value(QLatin1String("work:book")).toMap();
-
+        workQuery.remove(QLatin1String("*"));
+        
         QVariantMap authorSubQuery = workQuery;
         authorSubQuery.insert(QLatin1String("author~="), QLatin1Char('*') + request().value + QLatin1Char('*'));
 
         QVariantMap editorSubquery = workQuery;
         editorSubquery.insert(QLatin1String("editor~="), QLatin1Char('*') + request().value + QLatin1Char('*'));
 
+        // create dummy namespaces so as not to clobber full request
+        // for books with multiple authors, for example, not doing
+        // this results in onlyone author being returned
         QVariantMap authorQuery = query;
-        authorQuery.insert(QLatin1String("work:book"), authorSubQuery);
+        authorQuery.insert(QLatin1String("work2:book"), authorSubQuery);
 
         QVariantMap editorQuery = query;
-        editorQuery.insert(QLatin1String("work:book"), editorSubquery);
+        editorQuery.insert(QLatin1String("work2:book"), editorSubquery);
 
         queries << authorQuery << editorQuery;
       }
@@ -655,21 +698,25 @@ QVariantList FreebaseFetcher::bookQueries() const {
 
     case ISBN:
       {
-        QVariantMap isbnQuery = query.value(QLatin1String("isbn")).toMap();
+        QVariantMap isbnQuery = query.value(QLatin1String("/media_common/cataloged_instance/isbn13")).toMap();
         // search for both ISBN10 and ISBN13
         QVariantList isbns;
         foreach(const QString& isbn, FieldFormat::splitValue(request().value)) {
           isbns << ISBNValidator::cleanValue(ISBNValidator::isbn10(isbn));
           isbns << ISBNValidator::cleanValue(ISBNValidator::isbn13(isbn));
         }
-        isbnQuery.insert(QLatin1String("isbn|="), isbns);
-        query.insert(QLatin1String("isbn"), isbnQuery);
+        isbnQuery.insert(QLatin1String("value|="), isbns);
+        //remove the wildcard and optional since the ISBN is known
+        isbnQuery.remove(QLatin1String("*"));
+        isbnQuery.remove(QLatin1String("optional"));
+        //insert with a different "namespace" to avoid clobbering the request
+        query.insert(QLatin1String("isbn:/media_common/cataloged_instance/isbn13"), isbnQuery);
       }
       queries << query;
       break;
 
     case LCCN:
-      query.insert(QLatin1String("LCCN"), LCCNValidator::formalize(request().value));
+      query.insert(QLatin1String("/media_common/cataloged_instance/lccn"), LCCNValidator::formalize(request().value));
       queries << query;
       break;
 
@@ -743,22 +790,23 @@ QVariantList FreebaseFetcher::movieQueries() const {
 
     case Person:
       {
+        // use a dummy namespace for all of these
         QVariantMap directorQuery = query;
-        directorQuery.insert(QLatin1String("directed_by~="), QLatin1Char('*') + request().value + QLatin1Char('*'));
+        directorQuery.insert(QLatin1String("b:directed_by~="), QLatin1Char('*') + request().value + QLatin1Char('*'));
 
         QVariantMap producerQuery = query;
-        producerQuery.insert(QLatin1String("produced_by~="), QLatin1Char('*') + request().value + QLatin1Char('*'));
+        producerQuery.insert(QLatin1String("b:produced_by~="), QLatin1Char('*') + request().value + QLatin1Char('*'));
 
         QVariantMap writerQuery = query;
-        writerQuery.insert(QLatin1String("written_by~="), QLatin1Char('*') + request().value + QLatin1Char('*'));
+        writerQuery.insert(QLatin1String("b:written_by~="), QLatin1Char('*') + request().value + QLatin1Char('*'));
 
         QVariantMap composerQuery = query;
-        composerQuery.insert(QLatin1String("music~="), QLatin1Char('*') + request().value + QLatin1Char('*'));
+        composerQuery.insert(QLatin1String("b:music~="), QLatin1Char('*') + request().value + QLatin1Char('*'));
 
         QVariantMap castSubquery;
         castSubquery.insert(QLatin1String("actor~="), QLatin1Char('*') + request().value + QLatin1Char('*'));
         QVariantMap castQuery = query;
-        // have to give it a dummy namespace so we get back all the actors
+        // have to give it a dummy namespace so we get back all the people
         castQuery.insert(QLatin1String("b:starring"), QVariantList() << castSubquery);
 
         queries << directorQuery << castQuery << producerQuery << writerQuery << composerQuery;
@@ -898,15 +946,52 @@ QVariantList FreebaseFetcher::boardGameQueries() const {
 
 FreebaseFetcher::ConfigWidget::ConfigWidget(QWidget* parent_, const FreebaseFetcher* fetcher_)
     : Fetch::ConfigWidget(parent_) {
-  QVBoxLayout* l = new QVBoxLayout(optionsWidget());
-  l->addWidget(new QLabel(i18n("This source has no options."), optionsWidget()));
-  l->addStretch();
+  QGridLayout* l = new QGridLayout(optionsWidget());
+  l->setSpacing(4);
+  l->setColumnStretch(1, 10);
+
+  int row = -1;
+  QLabel* al = new QLabel(i18n("Registration is required for accessing the %1 data source. "
+                               "If you agree to the terms and conditions, <a href='%2'>sign "
+                               "up for an account</a>, and enter your information below.",
+                                preferredName(),
+                                QLatin1String("http://wiki.freebase.com/wiki/How_to_obtain_an_API_key")),
+                          optionsWidget());
+  al->setOpenExternalLinks(true);
+  al->setWordWrap(true);
+  ++row;
+  l->addWidget(al, row, 0, 1, 2);
+  // richtext gets weird with size
+  al->setMinimumWidth(al->sizeHint().width());
+
+  QLabel* label = new QLabel(i18n("API key: "), optionsWidget());
+  l->addWidget(label, ++row, 0);
+
+  m_apiKeyEdit = new KLineEdit(optionsWidget());
+  connect(m_apiKeyEdit, SIGNAL(textChanged(const QString&)), SLOT(slotSetModified()));
+  l->addWidget(m_apiKeyEdit, row, 1);
+  QString w = i18n("The default Tellico key may be used, but searching may fail due to reaching access limits.");
+  label->setWhatsThis(w);
+  m_apiKeyEdit->setWhatsThis(w);
+  label->setBuddy(m_apiKeyEdit);
+
+  l->setRowStretch(++row, 10);
 
   // now add additional fields widget
   addFieldsWidget(FreebaseFetcher::allOptionalFields(), fetcher_ ? fetcher_->optionalFields() : QStringList());
+
+  if(fetcher_ && fetcher_->m_apiKey != QLatin1String(FREEBASE_API_KEY)) {
+    // only show the key if it is not the default Tellico one...
+    // that way the user is prompted to apply for their own
+    m_apiKeyEdit->setText(fetcher_->m_apiKey);
+  }
 }
 
-void FreebaseFetcher::ConfigWidget::saveConfigHook(KConfigGroup&) {
+void FreebaseFetcher::ConfigWidget::saveConfigHook(KConfigGroup& config_) {
+  QString apiKey = m_apiKeyEdit->text().trimmed();
+  if(!apiKey.isEmpty()) {
+    config_.writeEntry("API Key", apiKey);
+  }
 }
 
 QString FreebaseFetcher::ConfigWidget::preferredName() const {
